using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.ComponentModel;
using System.Windows.Forms;

namespace RegionEditor
{
    public class Regions
    {
        private List<Region> m_Felucca;
        private List<Region> m_Trammel;
        private List<Region> m_Ilshenar;
        private List<Region> m_Malas;
        private List<Region> m_Tokuno;

        private List<Region>[] m_AllRegions;
        private string[] m_RegionNames = new string[] { "Felucca", "Trammel", "Ilshenar", "Malas", "Tokuno" };

		public List<Region>[] AllRegions
		{
			get { return m_AllRegions; }
		}
	

        public List<Region> Tokuno
        {
            get { return m_Tokuno; }
            set { m_Tokuno = value; }
        }

        public List<Region> Malas
        {
            get { return m_Malas; }
            set { m_Malas = value; }
        }

        public List<Region> Ilshenar
        {
            get { return m_Ilshenar; }
            set { m_Ilshenar = value; }
        }

        public List<Region> Trammel
        {
            get { return m_Trammel; }
            set { m_Trammel = value; }
        }

        public List<Region> Felucca
        {
            get { return m_Felucca; }
            set { m_Felucca = value; }
        }

        public Regions()
        {
            m_Felucca = new List<Region>();
            m_Trammel = new List<Region>();
            m_Ilshenar = new List<Region>();
            m_Malas = new List<Region>();
            m_Tokuno = new List<Region>();

            m_AllRegions = new List<Region>[] { m_Felucca, m_Trammel, m_Ilshenar, m_Malas, m_Tokuno };
        }

        public static Regions Load( string filename )
        {
            try
            {
                XmlDocument dom = new XmlDocument();
                dom.Load( filename );

                Regions reg = new Regions();
                XmlElement xRegs = dom[ "ServerRegions" ];

                foreach( XmlElement xFacet in xRegs.SelectNodes( "Facet" ) )
                {
                    string name = null;

                    if ( !XmlSupport.ReadString( xFacet, "name", ref name ) )
                        continue;

                    List<Region> target = null;

                    switch ( name.ToLower() )
                    {
                        case "felucca" : target = reg.m_Felucca;
                            break;

                        case "trammel" : target = reg.m_Trammel;
                            break;

                        case "ilshenar" : target = reg.m_Ilshenar;
                            break;

                        case "malas" : target = reg.m_Malas;
                            break;

                        case "tokuno" : target = reg.m_Tokuno;
                            break;
                    }

                    foreach( XmlElement xRegion in xFacet.SelectNodes( "region" ) )
                    {
                        Region r = Region.LoadRegion( xRegion );
                        if ( r != null )
                            target.Add( r );
                    }
                }

                return reg;
            }
            catch ( Exception err )
            {
                System.Windows.Forms.MessageBox.Show( string.Format( "An error has occurred. It has been placed on your clipboard:\n\n{0}", err.ToString() ) );
                System.Windows.Forms.Clipboard.SetDataObject( err.ToString() );
            }

            return null;
        }

        public bool Save( string filename )
        {
            try
            {
                XmlDocument dom = new XmlDocument();
                XmlDeclaration decl = dom.CreateXmlDeclaration( "1.0", "utf-8", null );
                dom.AppendChild( decl );

                XmlElement sr = dom.CreateElement( "ServerRegions" );

                for ( int i = 0; i < m_AllRegions.Length; i++ )
                {
                    XmlElement facet = dom.CreateElement( "Facet" );
                    facet.SetAttribute( "name", m_RegionNames[i] );

                    foreach ( Region r in m_AllRegions[i] )
                    {
                        XmlElement xRegion = r.GetXml( dom );

                        if ( xRegion != null )
                            facet.AppendChild( xRegion );
                    }

                    sr.AppendChild( facet );
                }

                dom.AppendChild( sr );
                dom.Save( filename );
                return true;
            }
            catch ( Exception err )
            {
                System.Windows.Forms.MessageBox.Show( string.Format( "An error occurred. The error text has been placed on the clipboard:\n\n{0}", err.ToString() ) );
                System.Windows.Forms.Clipboard.SetDataObject( err.ToString() );
            }

            return false;
        }
    }

    public enum XmlBool
    {
        Unspecified,
        True,
        False
    }

    public class Region
    {
        public const int DefaultMinZ = sbyte.MinValue;
        public const int DefaultMaxZ = sbyte.MaxValue + 1;

        private string m_Type = string.Empty;
        private int m_Priority = 50;
        private string m_Name = string.Empty;
        private Point3D m_GoLocation;
        private MusicName m_Music = MusicName.Invalid;
        private int m_MinZ = DefaultMinZ;
        private int m_MaxZ = DefaultMaxZ;
        private string m_Rune = string.Empty;
        private XmlBool m_SmartNoHousing = XmlBool.Unspecified;
        private XmlBool m_LogoutDelayActive = XmlBool.Unspecified;
        private List<Rectangle3D> m_Area;
        private List<Region> m_SubRegions;
        private XmlBool m_GuardsDisabled;

		[ Description( "This field is used only on guarded region, specifies whether guards should be disabled on world load." ) ]
        public XmlBool GuardsDisabled
        {
            get { return m_GuardsDisabled; }
            set { m_GuardsDisabled = value; }
        }

		[ Browsable( false ) ]
        public List<Region> SubRegions
        {
            get { return m_SubRegions; }
            set { m_SubRegions = value; }
        }

		[ Browsable( false ) ]
        public List<Rectangle3D> Area
        {
            get { return m_Area; }
            set { m_Area = value; }
        }

		[ Description( "Specifies whether players should experience a delay when logging out. Set to false for inn regions." ) ]
        public XmlBool LogoutDelayActive
        {
            get { return m_LogoutDelayActive; }
            set { m_LogoutDelayActive = value; }
        }

		[ Description( "Specifies whether the region should use RunUO's Smart No Housing algorithm as opposed to the OSI no housing system." ) ]
        public XmlBool SmartNoHousing
        {
            get { return m_SmartNoHousing; }
            set { m_SmartNoHousing = value; }
        }

		[ Description( "Specifies the name of runes marked in this region." ) ]
        public string Rune
        {
            get { return m_Rune; }
            set { m_Rune = value; }
        }

		[ Description( "Specifies the minimum Z level for the rectangles in this region." ) ]
        public int MaxZ
        {
            get { return m_MaxZ; }
            set
			{
				m_MaxZ = value;
				if ( m_MaxZ > DefaultMaxZ )
					m_MaxZ = DefaultMaxZ;
			}
        }
	
		[ Description( "Specifies the maximum Z level for the rectangles in this region." ) ]
        public int MinZ
        {
            get { return m_MinZ; }
            set
			{
				m_MinZ = value;

				if ( m_MinZ < DefaultMinZ )
					m_MinZ = DefaultMinZ;
			}
        }

		[ Description( "Sets the music for the region. Use Invalid not to specify any music." ) ]
        public MusicName Music
        {
            get { return m_Music; }
            set { m_Music = value; }
        }
	
		[ Description( "This is the location that will be used as target when using the [Go command." ) ]
        public Point3D GoLocation
        {
            get { return m_GoLocation; }
            set { m_GoLocation = value; }
        }

		[ Description( "The name of the region" ) ]
        public string Name
        {
            get { return m_Name; }
            set { m_Name = value; }
        }

		[ Description( "The priority of the region, 50 is the default value." ) ]
        public int Priority
        {
            get { return m_Priority; }
            set { m_Priority = value; }
        }

		[ Description( "Specifies the Type of the region, for automatic instantiation." ) ]
        public string Type
        {
            get { return m_Type; }
            set { m_Type = value; }
        }

        public Region()
        {
            m_Area = new List<Rectangle3D>();
			m_SubRegions = new List<Region>();
        }

        public static Region LoadRegion( XmlElement xml )
        {
            Region r = new Region();

            XmlSupport.ReadString( xml, "name", ref r.m_Name );
            XmlSupport.ReadInt32( xml, "priority", ref r.m_Priority );
            XmlSupport.ReadString( xml, "type", ref r.m_Type );

            XmlElement zrange = xml["zrange"];
            XmlSupport.ReadInt32( zrange, "min", ref r.m_MinZ, false );
            XmlSupport.ReadInt32( zrange, "max", ref r.m_MaxZ, false );

            foreach ( XmlElement xmlRect in xml.SelectNodes( "rect" ) )
            {
                Rectangle3D rect = new Rectangle3D();
                if ( XmlSupport.ReadRectangle3D( xmlRect, r.m_MinZ, r.m_MaxZ, ref rect ) )
                    r.m_Area.Add( rect );
            }

            if ( !XmlSupport.ReadPoint3D( xml["go"], ref r.m_GoLocation, false ) && r.m_Area.Count > 0 )
            {
                Point3D start = r.m_Area[0].Start;
         
                Point3D end = r.m_Area[0].End;

                int x = start.X + ( end.X - start.X ) / 2;
                int y = start.Y + ( end.Y - start.Y ) / 2;

                r.m_GoLocation = new Point3D( x, y, 0 );
            }

            object oMusic = MusicName.Invalid;
            XmlSupport.ReadEnum( xml["music"], "name", typeof( MusicName ), ref oMusic, false );
            r.m_Music = ( MusicName ) oMusic;

            // Begin other data

            // <rune name="Haven City" />
            XmlSupport.ReadString( xml["rune"], "name", ref r.m_Rune );
            // <smartNoHousing active="true" />
            XmlSupport.ReadXmlBool( xml["smartNoHousing"], "active", ref r.m_SmartNoHousing );
            // <logoutDelay active="false" />
            XmlSupport.ReadXmlBool( xml["logoutDelay"], "active", ref r.m_LogoutDelayActive );
            // <guards disabled="true" />
            XmlSupport.ReadXmlBool( xml["guards"], "disabled", ref r.m_GuardsDisabled );

            // TODO : Spawning

            // Subregions
            foreach ( XmlElement xRegion in xml.SelectNodes( "region" ) )
            {
                Region sub = Region.LoadRegion( xRegion );

                if ( sub != null )
                    r.m_SubRegions.Add( sub );
            }

            return r;
        }

        public XmlElement GetXml( XmlDocument dom )
        {
            XmlElement x = dom.CreateElement( "region" );

            if ( m_Name != string.Empty )
                x.SetAttribute( "name", m_Name );

            if ( m_Priority != 50 )
                x.SetAttribute( "priority", m_Priority.ToString() );

            if ( m_Type != string.Empty )
                x.SetAttribute( "type", m_Type );

            if ( m_MinZ != DefaultMinZ || m_MaxZ != DefaultMaxZ )
            {
                XmlElement xRange = dom.CreateElement( "zrange" );

                if ( m_MinZ != DefaultMinZ )
                    xRange.SetAttribute( "min", m_MinZ.ToString() );
                if ( m_MaxZ != DefaultMaxZ )
                    xRange.SetAttribute( "max", m_MaxZ.ToString() );

                x.AppendChild( xRange );
            }

            foreach ( Rectangle3D rect in m_Area )
            {
                XmlElement xRect = dom.CreateElement( "rect" );
                xRect.SetAttribute( "x", rect.Start.X.ToString() );
                xRect.SetAttribute( "y", rect.Start.Y.ToString() );
                xRect.SetAttribute( "width", rect.Width.ToString() );
                xRect.SetAttribute( "height", rect.Height.ToString() );

                if ( rect.Start.Z != DefaultMinZ || rect.Start.Z != m_MinZ )
                    xRect.SetAttribute( "zmin", rect.Start.Z.ToString() );
                if ( rect.End.Z != DefaultMaxZ || rect.End.Z != m_MaxZ )
                    xRect.SetAttribute( "zmax", rect.End.Z.ToString() );

                x.AppendChild( xRect );
            }

            if ( m_Music != MusicName.Invalid )
            {
                XmlElement xMusic = dom.CreateElement( "music" );
                xMusic.SetAttribute( "name", m_Music.ToString() );
                x.AppendChild( xMusic );
            }

            // Begin other data

            // <rune name="Haven City" />
            if ( m_Rune != null && m_Rune != string.Empty )
            {
                XmlElement xRune = dom.CreateElement( "rune" );
                xRune.SetAttribute( "name", m_Rune );
                x.AppendChild( xRune );
            }

            // <smartNoHousing active="true" />
            if ( m_SmartNoHousing != XmlBool.Unspecified )
            {
                XmlElement xHouse = dom.CreateElement( "smartNoHousing" );
                xHouse.SetAttribute( "active", m_SmartNoHousing.ToString().ToLower() );
                x.AppendChild( xHouse );
            }

            // <logoutDelay active="false" />
            if ( m_LogoutDelayActive != XmlBool.Unspecified )
            {
                XmlElement xLogout = dom.CreateElement( "logoutDelay" );
                xLogout.SetAttribute( "active", m_LogoutDelayActive.ToString().ToLower() );
                x.AppendChild( xLogout );
            }

            // <guards disabled="true" />
            if ( m_GuardsDisabled != XmlBool.Unspecified )
            {
                XmlElement xGuards = dom.CreateElement( "guards" );
                xGuards.SetAttribute( "disabled", m_GuardsDisabled.ToString().ToLower() );
                x.AppendChild( xGuards );
            }

            // TODO : Spawning

            // Subregions
            foreach ( Region sub in m_SubRegions )
            {
                XmlElement xSub = sub.GetXml( dom );
                x.AppendChild( xSub );
            }

            return x;
        }

		internal TreeNode GetTreeNode()
		{
			TreeNode node = new TreeNode( m_Name );

			if ( node.Text == string.Empty )
				node.Text = "Unnamed";

			node.Tag = this;

			foreach ( Region sub in m_SubRegions )
				node.Nodes.Add( sub.GetTreeNode() );

			return node;
		}
	}
}
