using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Box;
using Box.MapViewer;

namespace RegionEditor
{
	public partial class RegionEditorForm : Form
	{
		private Color RectColor
		{
			get { return Color.Blue; }
		}

		private Color RectFillColor
		{
			get { return Color.FromArgb( 80, RectColor ); }
		}

		private Color SelRectColor
		{
			get { return Color.White; }
		}

		private Color SelRectFillColor
		{
			get { return Color.FromArgb( 80, SelRectColor ); }
		}

		private Color DrawRectColor
		{
			get { return Color.Green; }
		}

		private Color DrawRectFillColor
		{
			get { return Color.FromArgb( 80, DrawRectColor ); }
		}

		public RegionEditorForm()
		{
			InitializeComponent();
			m_Regions = new Regions();
			cmbFacet.SelectedIndex = 0;
			m_DrawRectangles = new Dictionary<Rectangle3D, MapRectangle>();
		}

		private Regions m_Regions;
		private bool m_Modified = false;
		private string m_FileName;
		private List<Region> m_Facet;
		private Region m_Region;
		private Dictionary<Rectangle3D, MapRectangle> m_DrawRectangles;

		private Region SelectedRegion
		{
			set
			{
				if ( value == null )
					tabControl1.Enabled = false;
				else
					tabControl1.Enabled = true;

				m_Region = value;
				UpdateRegion();
			}
		}

		private void UpdateRegion()
		{
			if ( m_Region == null )
				return;

			propRegion.SelectedObject = m_Region;

			// Update rectangles
			SelectedRectangle = null;
			map.RemoveAllDrawObjects();
			m_DrawRectangles.Clear();

			lstRect.BeginUpdate();
			lstRect.Items.Clear();

			// Generate and draw all rectangles
			foreach ( Rectangle3D rect in m_Region.Area )
			{
				lstRect.Items.Add( rect );
				MapRectangle mr = new MapRectangle(
					new Rectangle( rect.X, rect.Y, rect.Width, rect.Height ),
					map.Map,
					RectColor,
					RectFillColor );
				m_DrawRectangles[rect] = mr;
				map.AddDrawObject( mr );
			}

			lstRect.EndUpdate();

			if ( m_Region.Area.Count > 0 )
			{
				lstRect.SelectedItem = m_Region.Area[0];
				SelectedRectangle = m_Region.Area[0];
			}

			// Move view to the go location, or first rect if available
			if ( m_Region.GoLocation != Point3D.Zero )
			{
				map.Center = new Point( m_Region.GoLocation.X, m_Region.GoLocation.Y );

				map.AddDrawObject( new Box.MapViewer.MapCircle( 4, map.Center, map.Map, Color.Yellow ) );
				map.AddDrawObject( new Box.MapViewer.MapCross( 6, Color.Yellow, map.Center, map.Map ) );
			}
			else if ( m_Region.Area.Count > 0 )
			{
				Rectangle3D r = m_Region.Area[0];

				int x = r.X + r.Width / 2;
				int y = r.Y + r.Height / 2;
				map.Center = new Point( x, y );
			}
		}

		private bool CheckModified()
		{
			if ( !m_Modified )
				return true;

			DialogResult result = MessageBox.Show(
				"The current document hasn't been saved. Do you wish to save it first?",
				"Document not saved",
				MessageBoxButtons.YesNoCancel );

			if ( result == DialogResult.Cancel )
				return false;
			else if ( result == DialogResult.No )
				return true;

			return Save();
		}

		private bool Save()
		{
			if ( m_FileName == null )
				return SaveAs();

			if ( m_Regions.Save( m_FileName ) )
			{
				m_Modified = false;
				return true;
			}

			return false;
		}

		private bool SaveAs()
		{
			if ( SaveFile.ShowDialog() == DialogResult.OK )
			{
				m_FileName = SaveFile.FileName;
				return Save();
			}

			return false;
		}

		private void miNew_Click( object sender, EventArgs e )
		{
			if ( CheckModified() )
			{
				m_Regions = new Regions();
				m_Modified = false;
				cmbFacet.SelectedIndex = 0;
				m_Facet = m_Regions.Felucca;
				map.Map = Maps.Felucca;
				RefreshView();
			}
		}

		private void miOpen_Click( object sender, EventArgs e )
		{
			if ( CheckModified() && LoadFile() )
			{
				cmbFacet.SelectedIndex = 0;
				m_Facet = m_Regions.Felucca;
				map.Map = Maps.Felucca;
				RefreshView();
			}
		}

		private bool LoadFile()
		{
			if ( OpenFile.ShowDialog() == DialogResult.OK )
			{
				Regions reg = Regions.Load( OpenFile.FileName );

				if ( reg != null )
				{
					m_Regions = reg;
					m_FileName = OpenFile.FileName;
					m_Modified = false;
					return true;
				}
			}

			return false;
		}

		private void miSaveAs_Click( object sender, EventArgs e )
		{
			SaveAs();
		}

		private void miSave_Click( object sender, EventArgs e )
		{
			Save();
		}

		private void cmbFacet_SelectedIndexChanged( object sender, EventArgs e )
		{
			m_Facet = m_Regions.AllRegions[cmbFacet.SelectedIndex];
			map.Map = ( Maps ) cmbFacet.SelectedIndex;
			RefreshView();
		}
		private void RefreshView()
		{
			tree.BeginUpdate();

			tree.Nodes.Clear();

			foreach ( Region region in m_Facet )
			{
				tree.Nodes.Add( region.GetTreeNode() );
			}

			tree.EndUpdate();
		}

		private void tree_AfterSelect( object sender, TreeViewEventArgs e )
		{
			Region r = e.Node.Tag as Region;

			SelectedRegion = r;
		}

		private void propRegion_PropertyValueChanged( object s, PropertyValueChangedEventArgs e )
		{
			m_Modified = true;

			if ( e.ChangedItem.PropertyDescriptor.Name == "Name" )
			{
				if ( m_Region.Name != string.Empty )
					tree.SelectedNode.Text = m_Region.Name;
				else
					tree.SelectedNode.Text = "Unnamed";
			}
		}

		private void bNewRegion_Click( object sender, EventArgs e )
		{
			Region r = new Region();
			m_Facet.Add( r );
			TreeNode node = r.GetTreeNode();
			tree.Nodes.Add( node );
			tree.SelectedNode = node;
			propRegion.Focus();
		}

		private void toolStripButton1_Click( object sender, EventArgs e )
		{
			if ( m_Region == null )
				return;

			Region r = new Region();
			TreeNode node = r.GetTreeNode();
			m_Region.SubRegions.Add( r );
			tree.SelectedNode.Nodes.Add( node );
			tree.SelectedNode = node;
			propRegion.Focus();
		}

		private void toolStripButton2_Click( object sender, EventArgs e )
		{
			if ( MessageBox.Show( "Are you sure you wish to delete the selected region and all of its subregions?",
				"Deleting a region", MessageBoxButtons.YesNo ) == DialogResult.Yes )
			{
				TreeNode parent = tree.SelectedNode.Parent;

				if ( parent == null )
					m_Facet.Remove( m_Region );
				else
				{
					Region r = parent.Tag as Region;
					r.SubRegions.Remove( m_Region );
				}

				SelectedRegion = null;
				tree.Nodes.Remove( tree.SelectedNode );
			}
		}

		private void miZoomIn_Click( object sender, EventArgs e )
		{
			miXRay.Checked = map.XRayView;
			miViewStatics.Checked = map.DrawStatics;
		}

		private void zoomInToolStripMenuItem_Click( object sender, EventArgs e )
		{
			map.ZoomIn();
		}

		private void miZoomOut_Click( object sender, EventArgs e )
		{
			map.ZoomOut();
		}

		private void miViewStatics_Click( object sender, EventArgs e )
		{
			map.DrawStatics = !map.DrawStatics;
		}

		private void miXRay_Click( object sender, EventArgs e )
		{
			map.XRayView = !map.XRayView;
		}

		#region Rectangles Drawing

		private Rectangle3D m_SelectedRectangle;

		private Rectangle3D SelectedRectangle
		{
			get { return m_SelectedRectangle; }
			set
			{
				if ( m_SelectedRectangle == value )
					return;

				// Revert selected rectangle to normal view
				if ( m_SelectedRectangle != null )
				{
					MapRectangle drawrect = m_DrawRectangles[m_SelectedRectangle];
					drawrect.Color = RectColor;
					drawrect.FillColor = RectFillColor;
					map.Refresh();
				}

				m_SelectedRectangle = value;

				if ( m_SelectedRectangle != null )
				{
					MapRectangle drawrect = m_DrawRectangles[ m_SelectedRectangle];
					drawrect.Color = SelRectColor;
					drawrect.FillColor = SelRectFillColor;
					map.Refresh();
				}

				propRect.SelectedObject = m_SelectedRectangle;
			}
		}

		#endregion

		private void lstRect_SelectedIndexChanged( object sender, EventArgs e )
		{
			Rectangle3D rect = lstRect.SelectedItem as Rectangle3D;
			SelectedRectangle = rect;
			FocusOnRect( rect );
		}

		private void FocusOnRect( Rectangle3D rect )
		{
			if ( rect == null )
				return;

			int x = rect.X + rect.Width / 2;
			int y = rect.Y + rect.Height / 2;
			map.Center = new Point( x, y );
		}

		private Rectangle3D Find( MapRectangle mr )
		{
			foreach( Rectangle3D r in m_DrawRectangles.Keys )
				if ( m_DrawRectangles[r] == mr )
					return r;
			return null;
		}

		private void RefreshRectanglesList()
		{
			lstRect.BeginUpdate();
			lstRect.Items.Clear();
			foreach ( Rectangle3D r in m_Region.Area )
				lstRect.Items.Add( r );
			lstRect.EndUpdate();
		}

		private void propRect_PropertyValueChanged( object s, PropertyValueChangedEventArgs e )
		{
			Rectangle3D rect = propRect.SelectedObject as Rectangle3D;
			MapRectangle mr = m_DrawRectangles[rect];
			mr.Rectangle = new Rectangle( rect.X, rect.Y, rect.Width, rect.Height );
			map.Refresh();

			RefreshRectanglesList();
			lstRect.SelectedItem = rect;			
		}

		private bool m_SetGo = false;
		private Point m_DragStart = Point.Empty;
		private bool m_Drawing = false;
		private bool m_Dragging = false;
		private MapRectangle m_DrawingRect;

		private void map_MouseDown( object sender, MouseEventArgs e )
		{
			if ( e.Button != MouseButtons.Left )
				return;

			MapRectangle rect = map.FindDrawObject( e.Location, 0 ) as MapRectangle;
			if ( rect != null )
			{
				Rectangle3D r = Find( rect );

				if ( r != null && r == SelectedRectangle )
				{
					// Unselect
					lstRect.SelectedItem = null;
					SelectedRectangle = null;
				}
				else if ( r != null )
				{
					// Select
					lstRect.SelectedItem = r;
					SelectedRectangle = r;
				}
			}

			if ( m_SetGo )
			{
				// TODO: Set go location
			}
			else
			{
				m_DragStart = map.ControlToMap( e.Location );

				if ( SelectedRectangle != null )
				{
					m_Dragging = true;
				}
				else
					m_Drawing = true;
			}
		}

		private void map_MouseMove( object sender, MouseEventArgs e )
		{
			if ( m_SetGo )
				return;

			if ( e.Button == MouseButtons.Left )
			{
				if ( m_Dragging )
				{
					Point p = map.ControlToMap( e.Location );

					if ( p == m_DragStart )
						return;

					int xdelta = p.X - m_DragStart.X;
					int ydelta = p.Y - m_DragStart.Y;

					SelectedRectangle.X += xdelta;
					SelectedRectangle.Y += ydelta;

					MapRectangle rect = m_DrawRectangles[SelectedRectangle];
					rect.Rectangle = new Rectangle( SelectedRectangle.X, SelectedRectangle.Y, SelectedRectangle.Width, SelectedRectangle.Height );
					map.Refresh();
					RefreshRectanglesList();
					propRect.Refresh();
					m_DragStart = p;
				}
				else if ( m_Drawing )
				{
					Point p = map.ControlToMap( e.Location );

					if ( m_DrawingRect == null )
					{
						m_DrawingRect = new MapRectangle(
							new Rectangle( m_DragStart.X, m_DragStart.Y, p.X - m_DragStart.X, p.Y - m_DragStart.Y ),
							map.Map,
							DrawRectColor,
							DrawRectFillColor );
						map.DrawObjects.Add( m_DrawingRect );
					}
					else
					{
						m_DrawingRect.Rectangle = new Rectangle( m_DragStart.X, m_DragStart.Y, p.X - m_DragStart.X, p.Y - m_DragStart.Y );
						map.Refresh();
					}
				}
			}
		}

		private void map_MouseUp( object sender, MouseEventArgs e )
		{
		}

	}
}
