using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace RegionEditor
{
	public partial class SpawnGroupsEditor : Form
	{
		public SpawnGroupsEditor()
		{
			InitializeComponent();
			m_Groups = new SpawnGroups();
		}

		private bool m_Modified = false;
		private SpawnGroups m_Groups;
		private string m_FileName;
		private SpawnGroup m_SelectedGroup;

		private void menuButton_ButtonClick( object sender, EventArgs e )
		{
			menuButton.ShowDropDown();
		}

		/// <summary>
		/// Verifies whether the current document has been modified.
		/// Should be called before any destructive action on the current data.
		/// </summary>
		/// <returns>True whether the user has agreed to proceed with the action</returns>
		private bool CheckModified()
		{
			if ( !m_Modified )
				return true;

			DialogResult result = MessageBox.Show( "The current document has been modified but not saved. Do you wish to save it before proceeding?",
				"Document not saved", MessageBoxButtons.YesNoCancel );

			if ( result == DialogResult.Cancel )
				return false;
			else if ( result == DialogResult.No )
				return true;

			// Do save as
			return SaveAs();

		}

		/// <summary>
		/// Performs the Save As action, showing the user a save dialog
		/// </summary>
		/// <returns>True if the file has been saved, false otherwise</returns>
		private bool SaveAs()
		{
			if ( SaveFile.ShowDialog() == DialogResult.OK )
			{
				bool ok = m_Groups.Save( SaveFile.FileName );

				if ( ok )
				{
					m_Modified = false;
					m_FileName = SaveFile.FileName;
				}

				return ok;
			}
			else
				return false;
		}

		/// <summary>
		/// Performs the save action on the current filename
		/// </summary>
		/// <returns></returns>
		private bool Save()
		{
			if ( m_FileName == null )
				return SaveAs();

			bool ok = m_Groups.Save( m_FileName );

			if ( ok )
				m_Modified = false;

			return ok;
		}

		/// <summary>
		/// Creates a new file
		/// </summary>
		private void New()
		{
			if ( CheckModified() )
			{
				m_FileName = null;
				m_Modified = false;
				m_Groups = new SpawnGroups();
				RefreshView();
			}
		}

		/// <summary>
		/// Loads a new file
		/// </summary>
		/// <returns>True if the file has been loaded correctly</returns>
		private bool PerformLoad()
		{
			if ( CheckModified() && OpenFile.ShowDialog() == DialogResult.OK )
			{
				SpawnGroups gps = SpawnGroups.Load( OpenFile.FileName );

				if ( gps != null )
				{
					m_Groups = gps;
					m_Modified = false;
					m_FileName = OpenFile.FileName;
					RefreshView();
				}
			}
			
			return false;
		}

		/// <summary>
		/// Refreshes the view according to the current groups
		/// </summary>
		private void RefreshView()
		{
			txNewGroup.Clear();

			m_Groups.Spawns.Sort();

			cmbGroups.BeginUpdate();
			cmbGroups.Items.Clear();

			foreach ( SpawnGroup grp in m_Groups.Spawns )
			{
				cmbGroups.Items.Add( grp );
			}

			cmbGroups.EndUpdate();

			m_SelectedGroup = null;

			FixSelectedGroup();
		}

		private void miNew_Click( object sender, EventArgs e )
		{
			New();
		}

		private void miOpen_Click( object sender, EventArgs e )
		{
			PerformLoad();
		}

		private void miSave_Click( object sender, EventArgs e )
		{
			Save();
		}

		private void miSaveAs_Click( object sender, EventArgs e )
		{
			SaveAs();
		}

		private void miExit_Click( object sender, EventArgs e )
		{
			Close();
		}

		private void SpawnGroupsEditor_FormClosing( object sender, FormClosingEventArgs e )
		{
			if ( m_Modified && !CheckModified() )
				e.Cancel = true;
		}

		private void toolStripTextBox1_TextChanged( object sender, EventArgs e )
		{
			string newName = txNewGroup.Text;

			bool ok = newName != null && newName != string.Empty;

			foreach( SpawnGroup g in m_Groups.Spawns )
				if ( g.Name.ToLower() == newName.ToLower() )
				{
					ok = false;
					break;
				}

			bAddGroup.Enabled = ok;
		}

		private void cmbGroups_SelectedIndexChanged( object sender, EventArgs e )
		{
			m_SelectedGroup = cmbGroups.SelectedItem as SpawnGroup;
			FixSelectedGroup();
		}

		private void FixSelectedGroup()
		{
			if ( m_SelectedGroup == null )
			{
				txName.Clear();
				txNewObject.Clear();
				txDelete.Clear();
				grpbox.Text = string.Empty;
				grpbox.Enabled = false;
			}
			else
			{
				txName.Text = m_SelectedGroup.Name;
				txNewObject.Clear();
				txDelete.Clear();
				grpbox.Text = m_SelectedGroup.Name;
				grpbox.Enabled = true;
				bDelete.Enabled = false;
				bAddObject.Enabled = false;
				bChangeName.Enabled = false;

				lst.BeginUpdate();
				lst.Items.Clear();
				foreach ( string s in m_SelectedGroup.Items )
					lst.Items.Add( s );
				lst.EndUpdate();
			}
		}

		private bool CheckName( string name, SpawnGroup grp )
		{
			foreach ( SpawnGroup g in m_Groups.Spawns )
			{
				if ( g != grp && name.ToLower() == g.Name.ToLower() )
					return false;
			}

			return true;
		}

		private void txName_TextChanged( object sender, EventArgs e )
		{
			if ( m_SelectedGroup == null )
				return;

			string newname = txName.Text;

			if ( CheckName( newname, m_SelectedGroup ) && newname != string.Empty && newname != m_SelectedGroup.Name )
				bChangeName.Enabled = true;
			else
				bChangeName.Enabled = false;			
		}

		private void bChangeName_Click( object sender, EventArgs e )
		{
			m_SelectedGroup.Name = txName.Text;
			grpbox.Text = m_SelectedGroup.Name;

			m_Groups.Spawns.Sort();

			cmbGroups.BeginUpdate();
			cmbGroups.Items.Clear();
			foreach ( SpawnGroup g in m_Groups.Spawns )
				cmbGroups.Items.Add( g );
			cmbGroups.EndUpdate();

			cmbGroups.SelectedItem = m_SelectedGroup;			

			bChangeName.Enabled = false;
			m_Modified = true;
		}

		private void txNewObject_TextChanged( object sender, EventArgs e )
		{
			bAddObject.Enabled = txNewObject.Text != string.Empty
			&& txNewObject.Text.IndexOf( ' ' ) == -1;
		}

		private void lst_SelectedIndexChanged( object sender, EventArgs e )
		{
			if ( lst.SelectedItem == null )
			{
				txDelete.Clear();
				bDelete.Enabled = false;
			}
			else
			{
				txDelete.Text = lst.SelectedItem as string;
				bDelete.Enabled = true;
			}
		}

		private void bDelete_Click( object sender, EventArgs e )
		{
			if ( lst.SelectedItem == null )
				return;

			txDelete.Clear();
			bDelete.Enabled = false;
			m_SelectedGroup.Items.Remove( lst.SelectedItem as string );
			lst.Items.Remove( lst.SelectedItem );
			m_Modified = true;
			
		}

		private void lst_DragEnter( object sender, DragEventArgs e )
		{
			if ( e.Data.GetDataPresent( typeof( string ) ) )
				e.Effect = DragDropEffects.Copy;
			else
				e.Effect = DragDropEffects.None;
		}

		private void lst_DragDrop( object sender, DragEventArgs e )
		{
			string newObject = e.Data.GetData( typeof( string ) ) as string;

			if ( newObject != null && newObject != string.Empty && newObject.IndexOf( ' ' ) == -1 )
			{
				m_SelectedGroup.Items.Add( newObject );
				lst.Items.Add( newObject );
				m_Modified = true;
			}
		}

		private void bAddObject_Click( object sender, EventArgs e )
		{
			string newobj = txNewObject.Text;

			m_SelectedGroup.Items.Add( newobj );
			lst.Items.Add( newobj );

			txNewObject.Clear();
			bAddObject.Enabled = false;
			m_Modified = true;
			txNewObject.Focus();
		}

		private void bAddGroup_Click( object sender, EventArgs e )
		{
			SpawnGroup grp = new SpawnGroup();
			grp.Name = txNewGroup.Text;
			m_Groups.Spawns.Add( grp );
			m_Groups.Spawns.Sort();

			cmbGroups.BeginUpdate();
			cmbGroups.Items.Clear();
			foreach ( SpawnGroup g in m_Groups.Spawns )
				cmbGroups.Items.Add( g );
			cmbGroups.EndUpdate();
			cmbGroups.SelectedItem = grp;
			m_SelectedGroup = grp;
			m_Modified = true;
		}

		private void txNewGroup_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter && bAddGroup.Enabled )
				bAddGroup.PerformClick();
		}

		private void txName_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter && bChangeName.Enabled )
				bChangeName.PerformClick();
		}

		private void txNewObject_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Enter && bAddObject.Enabled )
				bAddObject.PerformClick();
		}
	}
}